==> beigesoft-bcommon-1.1.7-src.tar.bz2: src/main/java/org/beigesoft/handler/HndlI18nRequest.java <==
package org.beigesoft.handler;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.List;
import java.util.Map;
import java.util.Locale;

import org.beigesoft.model.IRequestData;
import org.beigesoft.persistable.Languages;
import org.beigesoft.persistable.DecimalSeparator;
import org.beigesoft.persistable.DecimalGroupSeparator;
import org.beigesoft.persistable.LangPreferences;
import org.beigesoft.log.ILogger;
import org.beigesoft.service.ISrvDatabase;
import org.beigesoft.service.ISrvOrm;

/**
 * <p>It handles request internationalization.</p>
 *
 * @param <RS> platform dependent RDBMS recordset
 * @author Yury Demidenko
 */
public class HndlI18nRequest<RS> implements IHandlerRequestDch {

  /**
   * <p>Logger.</p>
   **/
  private ILogger logger;

  /**
   * <p>Database service.</p>
   */
  private ISrvDatabase<RS> srvDatabase;

  /**
   * <p>ORM service.</p>
   */
  private ISrvOrm<RS> srvOrm;

  /**
   * <p>Cached Language Preferences.</p>
   */
  private List<LangPreferences> langPreferences;

  /**
   * <p>Cached languages.</p>
   */
  private List<Languages> languages;

  /**
   * <p>Cached decimal separators.</p>
   */
  private List<DecimalSeparator> decimalSeparators;

  /**
   * <p>Cached decimal group separators.</p>
   */
  private List<DecimalGroupSeparator> decimalGroupSeparators;

  /**
   * <p>Additional I18n Request Handler - setting I18N related
   * variables for current task,
   * e.g. it's uses only 0 decimal places in the current request,
   * so JS will invokes only:
   * $('#'+ pTarget +' .autoNum0').autoNumeric('init', {mDec: '0'});.</p>
   */
  private IHandlerRequestDch additionalI18nReqHndl;

  /**
   * <p>Handle request. Synchronization whole code may hit performance,
   * so data is copied into temporary references in synchronize block.
   * Any way it's hardly ever happens changing languages features
   * in database.</p>
   * @param pReqVars Request scoped variables
   * @param pRequestData Request Data
   * @throws Exception - an exception
   */
  @Override
  public final void handle(final Map<String, Object> pReqVars,
    final IRequestData pRequestData) throws Exception {
    List<LangPreferences> lpfsTmp;
    List<Languages> lgsTmp;
    List<DecimalSeparator> dssTmp;
    List<DecimalGroupSeparator> dgssTmp;
    boolean isFirstReq = false;
    synchronized (this) {
      if (this.langPreferences == null) {
        try {
          isFirstReq = true;
          this.logger.info(null, HndlI18nRequest.class,
            "Refreshing I18N data...");
          this.srvDatabase.setIsAutocommit(false);
          this.srvDatabase.setTransactionIsolation(ISrvDatabase
            .TRANSACTION_READ_UNCOMMITTED);
          this.srvDatabase.beginTransaction();
          List<LangPreferences> lpfs = this.srvOrm.retrieveList(pReqVars,
            LangPreferences.class);
          List<Languages> lgs = this.srvOrm.retrieveList(pReqVars,
            Languages.class);
          List<DecimalSeparator> dss = this.srvOrm.retrieveList(pReqVars,
            DecimalSeparator.class);
          List<DecimalGroupSeparator> dgss = this.srvOrm.retrieveList(
            pReqVars, DecimalGroupSeparator.class);
          this.srvDatabase.commitTransaction();
          //assigning fully initialized data:
          this.decimalGroupSeparators = dgss;
          this.decimalSeparators = dss;
          this.languages = lgs;
          this.langPreferences = lpfs;
        } catch (Exception ex) {
          if (!this.srvDatabase.getIsAutocommit()) {
            this.srvDatabase.rollBackTransaction();
          }
          throw ex;
        } finally {
          this.srvDatabase.releaseResources();
        }
      }
      lpfsTmp = this.langPreferences;
      lgsTmp = this.languages;
      dssTmp = this.decimalSeparators;
      dgssTmp = this.decimalGroupSeparators;
    }
    LangPreferences lpf = revealLangPreferences(pRequestData, lgsTmp,
      dssTmp, dgssTmp, lpfsTmp, isFirstReq);
    String langDef = null;
    for (LangPreferences lpft : lpfsTmp) {
      if (lpft.getIsDefault()) {
        langDef = lpft.getLang().getItsId();
        break;
      }
    }
    if (lpfsTmp.size() == 0 || langDef == null) {
      this.logger.error(null, HndlI18nRequest.class,
        "There is no default languages preferences!");
      //I18N service is soft coded, so it will be at least braced messages
      //like [Ok]
      pReqVars.put("lang", "en");
      pReqVars.put("langDef", "en");
      pReqVars.put("localeCurrent", new Locale("en", "US"));
      pReqVars.put("dseparator", ".");
      pReqVars.put("dgseparator", ",");
      pReqVars.put("digitsInGroup", 3);
      return;
    }
    pReqVars.put("lang", lpf.getLang().getItsId());
    pReqVars.put("langDef", langDef);
    Locale locCurr;
    if (pRequestData.getLocale().getLanguage()
      .equals(lpf.getLang().getItsId())) {
      locCurr = pRequestData.getLocale();
    } else {
      locCurr = new Locale(lpf.getLang().getItsId());
    }
    pReqVars.put("localeCurrent", locCurr);
    pReqVars.put("dseparator", lpf.getDecimalSep().getItsId());
    pReqVars.put("digitsInGroup", lpf.getDigitsInGroup());
    String dseparatorv = lpf.getDecimalSep().getItsId();
    if ("space".equals(dseparatorv)) {
      dseparatorv = " ";
    }
    pReqVars.put("dseparatorv", dseparatorv);
    pReqVars.put("dgseparator", lpf.getDecimalGroupSep()
      .getItsId());
    String dgseparatorv = lpf.getDecimalGroupSep().getItsId();
    if ("space".equals(dgseparatorv)) {
      dgseparatorv = " ";
    }
    pReqVars.put("dgseparatorv", dgseparatorv);
    pReqVars.put("languages", lgsTmp);
    pReqVars.put("decimalSeparators", dssTmp);
    pReqVars.put("decimalGroupSeparators", dgssTmp);
    if (this.additionalI18nReqHndl != null) {
      this.additionalI18nReqHndl.handle(pReqVars, pRequestData);
    } else {
      pReqVars.put("quantityPrecision", Integer.valueOf(2));
      pReqVars.put("pricePrecision", Integer.valueOf(2));
      pReqVars.put("costPrecision", Integer.valueOf(2));
      pReqVars.put("RSmRound", "S");
      pReqVars.put("RSisUsePrecision0", Boolean.TRUE);
      pReqVars.put("RSisUsePrecision1", Boolean.TRUE);
      pReqVars.put("RSisUsePrecision2", Boolean.TRUE);
      pReqVars.put("RSisUsePrecision3", Boolean.TRUE);
      pReqVars.put("RSisUsePrecision4", Boolean.TRUE);
    }
  }

  /**
   * <p>Reveals language preference and adds preferences
   * into cookie if need.</p>
   * @param pRequestData Request Data
   * @param pLangs lang. list
   * @param pDecSeps dec.sep. list
   * @param pDecGrSeps dec.gr.sep. list
   * @param pLangPrefs lang.pref. list
   * @param pIsFirstReq if first request
   * @return lang. preferences
   * @throws Exception - an exception
   */
  public final LangPreferences revealLangPreferences(
    final IRequestData pRequestData, final List<Languages> pLangs,
      final List<DecimalSeparator> pDecSeps,
        final List<DecimalGroupSeparator> pDecGrSeps,
          final List<LangPreferences> pLangPrefs,
            final boolean pIsFirstReq) throws Exception {
    int detLev = this.logger.getDetailLevel();
    boolean isShowDebMsg = this.logger.getIsShowDebugMessages()
      && this.logger.getIsShowDebugMessagesFor(this.getClass());
    LangPreferences lpf = null;
    boolean needSetCookie = false;
    //check request changing preferences:
    String lang = pRequestData.getParameter("lang");
    String dseparator = pRequestData.getParameter("dseparator");
    String dgseparator = pRequestData.getParameter("dgseparator");
    String digitsInGroup = pRequestData.getParameter("digitsInGroup");
    if (pIsFirstReq || isShowDebMsg && detLev > 60000) {
      this.logger.debug(null, HndlI18nRequest.class,
        "Request's lang/dseparator/dgseparator/digitsInGroup: " + lang + "/"
          + dseparator + "/" + dgseparator + "/" + digitsInGroup);
    }
    if (lang == null || lang.length() == 0
      || dseparator == null || dseparator.length() == 0
        || dgseparator == null || dgseparator.length() == 0
          || digitsInGroup == null || digitsInGroup.length() == 0) {
      lang = null;
      dseparator = null;
      dgseparator = null;
      digitsInGroup = null;
    } else {
      needSetCookie = true;
    }
    if (dgseparator == null && dseparator == null && lang == null) {
      lang = pRequestData.getCookieValue("lang");
      dseparator = pRequestData.getCookieValue("dseparator");
      dgseparator = pRequestData.getCookieValue("dgseparator");
      digitsInGroup = pRequestData.getCookieValue("digitsInGroup");
      if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
        this.logger.debug(null, HndlI18nRequest.class,
          "Client's cookie lang/dseparator/dgseparator/digitsInGroup: " + lang
            + "/" + dseparator + "/" + dgseparator + "/" + digitsInGroup);
      }
    }
    if (dgseparator != null && dseparator != null && lang != null
      && digitsInGroup != null) {
      if (dgseparator.equals(dseparator)) {
        if (pIsFirstReq || isShowDebMsg) {
          this.logger.debug(null, HndlI18nRequest.class,
            "Separators are same!! dseparator/dgseparator: " + dseparator);
        }
      } else {
        //try to match from cookies or changed by user:
        lpf = new LangPreferences();
        lpf.setDigitsInGroup(Integer.parseInt(digitsInGroup));
        for (Languages ln : pLangs) {
          if (ln.getItsId().equals(lang)) {
            lpf.setLang(ln);
            break;
          }
        }
        if (lpf.getLang() == null) {
          lpf = null;
        } else {
          for (DecimalSeparator ds : pDecSeps) {
            if (ds.getItsId().equals(dseparator)) {
              lpf.setDecimalSep(ds);
              break;
            }
          }
          if (lpf.getDecimalSep() == null) {
            lpf = null;
          } else {
            for (DecimalGroupSeparator dgs : pDecGrSeps) {
              if (dgs.getItsId().equals(dgseparator)) {
                lpf.setDecimalGroupSep(dgs);
                break;
              }
            }
            if (lpf.getDecimalGroupSep() == null) {
              lpf = null;
            }
          }
        }
      }
    }
    if (lpf == null) {
      // try match client's locale, if not - default or the first:
      String ccountry = null;
      String clang = null;
      if (pRequestData.getLocale() != null) {
        ccountry = pRequestData.getLocale().getCountry();
        clang = pRequestData.getLocale().getLanguage();
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
            "Client prefers lang/country: " + clang + "/" + ccountry);
        }
      }
      LangPreferences lpfMf = null;
      LangPreferences lpfMl = null;
      LangPreferences lpfDef = null;
      for (LangPreferences lpft : pLangPrefs) {
        if (lpft.getCountry().getItsId().equals(ccountry)
          && lpft.getLang().getItsId().equals(clang)) {
          lpfMf = lpft;
          break;
        }
        if (lpft.getLang().getItsId().equals(clang)) {
          lpfMl = lpft;
        }
        if (lpft.getIsDefault()) {
          lpfDef = lpft;
        } else if (lpfDef == null) {
          lpfDef = lpft;
        }
      }
      if (lpfMf != null) {
        lpf = lpfMf;
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
        "Full match lang/dseparator/dgseparator/digitsInGroup: " + lpf.getLang()
      .getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
    + lpf.getDecimalGroupSep().getItsId() + "/" + lpf.getDigitsInGroup());
        }
      } else if (lpfMl != null) {
        lpf = lpfMl;
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
        "Lang match lang/dseparator/dgseparator/digitsInGroup: " + lpf.getLang()
      .getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
    + lpf.getDecimalGroupSep().getItsId() + "/" + lpf.getDigitsInGroup());
        }
      } else {
        lpf = lpfDef;
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
        "No match, default lang/dseparator/dgseparator/digitsInGroup: "
      + lpf.getLang().getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
    + lpf.getDecimalGroupSep().getItsId() + "/" + lpf.getDigitsInGroup());
        }
      }
      needSetCookie = true;
    }
    if (needSetCookie) {
      pRequestData.setCookieValue("digitsInGroup",
        lpf.getDigitsInGroup().toString());
      pRequestData.setCookieValue("lang", lpf.getLang().getItsId());
      pRequestData.setCookieValue("dseparator",
        lpf.getDecimalSep().getItsId());
      pRequestData.setCookieValue("dgseparator",
        lpf.getDecimalGroupSep().getItsId());
      if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
        this.logger.debug(null, HndlI18nRequest.class,
          "Cookie is set lang/dseparator/dgseparator: " + lpf.getLang()
            .getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
              + lpf.getDecimalGroupSep().getItsId());
      }
    }
    return lpf;
  }


  /**
   * <p>Handle data changed event.</p>
   * @throws Exception - an exception
   **/
  @Override
  public final synchronized void handleDataChanged() throws Exception {
    this.langPreferences = null;
    this.languages = null;
    this.decimalSeparators = null;
    this.decimalGroupSeparators = null;
    this.logger.info(null, HndlI18nRequest.class, "I18N changes are handled.");
    if (this.additionalI18nReqHndl != null) {
      this.additionalI18nReqHndl.handleDataChanged();
    }
  }

  //Simple getters and setters (their synchronization never hit performance):
  /**
   * <p>Geter for logger.</p>
   * @return ILogger
   **/
  public final synchronized ILogger getLogger() {
    return this.logger;
  }

  /**
   * <p>Setter for logger.</p>
   * @param pLogger reference
   **/
  public final synchronized void setLogger(final ILogger pLogger) {
    this.logger = pLogger;
  }

  /**
   * <p>Getter for srvDatabase.</p>
   * @return ISrvDatabase<RS>
   **/
  public final synchronized ISrvDatabase<RS> getSrvDatabase() {
    return this.srvDatabase;
  }

  /**
   * <p>Setter for srvDatabase.</p>
   * @param pSrvDatabase reference
   **/
  public final synchronized void setSrvDatabase(
    final ISrvDatabase<RS> pSrvDatabase) {
    this.srvDatabase = pSrvDatabase;
  }

  /**
   * <p>Getter for srvOrm.</p>
   * @return ISrvOrm<RS>
   **/
  public final synchronized ISrvOrm<RS> getSrvOrm() {
    return this.srvOrm;
  }

  /**
   * <p>Setter for srvOrm.</p>
   * @param pSrvOrm reference
   **/
  public final synchronized void setSrvOrm(final ISrvOrm<RS> pSrvOrm) {
    this.srvOrm = pSrvOrm;
  }

  /**
   * <p>Getter for additionalI18nReqHndl.</p>
   * @return IHandlerRequest
   **/
  public final IHandlerRequestDch getAdditionalI18nReqHndl() {
    return this.additionalI18nReqHndl;
  }

  /**
   * <p>Setter for additionalI18nReqHndl.</p>
   * @param pAdditionalI18nReqHndl reference
   **/
  public final void setAdditionalI18nReqHndl(
    final IHandlerRequestDch pAdditionalI18nReqHndl) {
    this.additionalI18nReqHndl = pAdditionalI18nReqHndl;
  }
}

==> beigesoft-bcommon-1.1.8-src.tar.bz2: src/main/java/org/beigesoft/handler/HndlI18nRequest.java <==
package org.beigesoft.handler;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.List;
import java.util.Map;
import java.util.Locale;

import org.beigesoft.model.IRequestData;
import org.beigesoft.persistable.Languages;
import org.beigesoft.persistable.DecimalSeparator;
import org.beigesoft.persistable.DecimalGroupSeparator;
import org.beigesoft.persistable.LangPreferences;
import org.beigesoft.log.ILogger;
import org.beigesoft.service.ISrvDatabase;
import org.beigesoft.service.ISrvOrm;

/**
 * <p>It handles request internationalization.</p>
 *
 * @param <RS> platform dependent RDBMS recordset
 * @author Yury Demidenko
 */
public class HndlI18nRequest<RS> implements IHandlerRequestDch {

  /**
   * <p>Logger.</p>
   **/
  private ILogger logger;

  /**
   * <p>Database service.</p>
   */
  private ISrvDatabase<RS> srvDatabase;

  /**
   * <p>ORM service.</p>
   */
  private ISrvOrm<RS> srvOrm;

  /**
   * <p>Cached Language Preferences.</p>
   */
  private List<LangPreferences> langPreferences;

  /**
   * <p>Cached languages.</p>
   */
  private List<Languages> languages;

  /**
   * <p>Cached decimal separators.</p>
   */
  private List<DecimalSeparator> decSeps;

  /**
   * <p>Cached decimal group separators.</p>
   */
  private List<DecimalGroupSeparator> decGrSeps;

  /**
   * <p>Additional I18n Request Handler - setting I18N related
   * variables for current task,
   * e.g. it's uses only 0 decimal places in the current request,
   * so JS will invokes only:
   * $('#'+ pTarget +' .autoNum0').autoNumeric('init', {mDec: '0'});.</p>
   */
  private IHandlerRequestDch additionalI18nReqHndl;

  /**
   * <p>Handle request. Synchronization whole code may hit performance,
   * so data is copied into temporary references in synchronize block.
   * Any way it's hardly ever happens changing languages features
   * in database.</p>
   * @param pReqVars Request scoped variables
   * @param pRequestData Request Data
   * @throws Exception - an exception
   */
  @Override
  public final void handle(final Map<String, Object> pReqVars,
    final IRequestData pRequestData) throws Exception {
    List<LangPreferences> lpfsTmp;
    List<Languages> lgsTmp;
    List<DecimalSeparator> dssTmp;
    List<DecimalGroupSeparator> dgssTmp;
    boolean isFirstReq = false;
    synchronized (this) {
      if (this.langPreferences == null) {
        try {
          isFirstReq = true;
          this.logger.info(null, HndlI18nRequest.class,
            "Refreshing I18N data...");
          this.srvDatabase.setIsAutocommit(false);
          this.srvDatabase.setTransactionIsolation(ISrvDatabase
            .TRANSACTION_READ_UNCOMMITTED);
          this.srvDatabase.beginTransaction();
          List<LangPreferences> lpfs = this.srvOrm.retrieveList(pReqVars,
            LangPreferences.class);
          List<Languages> lgs = this.srvOrm.retrieveList(pReqVars,
            Languages.class);
          List<DecimalSeparator> dss = this.srvOrm.retrieveList(pReqVars,
            DecimalSeparator.class);
          List<DecimalGroupSeparator> dgss = this.srvOrm.retrieveList(
            pReqVars, DecimalGroupSeparator.class);
          this.srvDatabase.commitTransaction();
          //assigning fully initialized data:
          this.decGrSeps = dgss;
          this.decSeps = dss;
          this.languages = lgs;
          this.langPreferences = lpfs;
        } catch (Exception ex) {
          if (!this.srvDatabase.getIsAutocommit()) {
            this.srvDatabase.rollBackTransaction();
          }
          throw ex;
        } finally {
          this.srvDatabase.releaseResources();
        }
      }
      lpfsTmp = this.langPreferences;
      lgsTmp = this.languages;
      dssTmp = this.decSeps;
      dgssTmp = this.decGrSeps;
    }
    LangPreferences lpf = revealLangPreferences(pRequestData, lgsTmp,
      dssTmp, dgssTmp, lpfsTmp, isFirstReq);
    String langDef = null;
    for (LangPreferences lpft : lpfsTmp) {
      if (lpft.getIsDefault()) {
        langDef = lpft.getLang().getItsId();
        break;
      }
    }
    if (lpfsTmp.size() == 0 || langDef == null) {
      this.logger.error(null, HndlI18nRequest.class,
        "There is no default languages preferences!");
      //I18N service is soft coded, so it will be at least braced messages
      //like [Ok]
      pReqVars.put("lang", "en");
      pReqVars.put("langDef", "en");
      pReqVars.put("localeCurr", new Locale("en", "US"));
      pReqVars.put("decSep", ".");
      pReqVars.put("decGrSep", ",");
      pReqVars.put("digInGr", 3);
      return;
    }
    pReqVars.put("lang", lpf.getLang().getItsId());
    pReqVars.put("langDef", langDef);
    Locale locCurr;
    if (pRequestData.getLocale().getLanguage()
      .equals(lpf.getLang().getItsId())) {
      locCurr = pRequestData.getLocale();
    } else {
      locCurr = new Locale(lpf.getLang().getItsId());
    }
    pReqVars.put("localeCurr", locCurr);
    pReqVars.put("decSep", lpf.getDecimalSep().getItsId());
    pReqVars.put("digInGr", lpf.getDigitsInGroup());
    String decSepv = lpf.getDecimalSep().getItsId();
    if ("space".equals(decSepv)) {
      decSepv = "\u00A0";
    }
    pReqVars.put("decSepv", decSepv);
    pReqVars.put("decGrSep", lpf.getDecimalGroupSep()
      .getItsId());
    String decGrSepv = lpf.getDecimalGroupSep().getItsId();
    if ("space".equals(decGrSepv)) {
      decGrSepv = "\u00A0";
    }
    pReqVars.put("decGrSepv", decGrSepv);
    pReqVars.put("languages", lgsTmp);
    pReqVars.put("decSeps", dssTmp);
    pReqVars.put("decGrSeps", dgssTmp);
    if (this.additionalI18nReqHndl != null) {
      this.additionalI18nReqHndl.handle(pReqVars, pRequestData);
    } else {
      pReqVars.put("quantityDp", Integer.valueOf(2));
      pReqVars.put("priceDp", Integer.valueOf(2));
      pReqVars.put("costDp", Integer.valueOf(2));
    }
  }

  /**
   * <p>Reveals language preference and adds preferences
   * into cookie if need.</p>
   * @param pRequestData Request Data
   * @param pLangs lang. list
   * @param pDecSeps dec.sep. list
   * @param pDecGrSeps dec.gr.sep. list
   * @param pLangPrefs lang.pref. list
   * @param pIsFirstReq if first request
   * @return lang. preferences
   * @throws Exception - an exception
   */
  public final LangPreferences revealLangPreferences(
    final IRequestData pRequestData, final List<Languages> pLangs,
      final List<DecimalSeparator> pDecSeps,
        final List<DecimalGroupSeparator> pDecGrSeps,
          final List<LangPreferences> pLangPrefs,
            final boolean pIsFirstReq) throws Exception {
    int detLev = this.logger.getDetailLevel();
    boolean isShowDebMsg = this.logger.getIsShowDebugMessages()
      && this.logger.getIsShowDebugMessagesFor(this.getClass());
    LangPreferences lpf = null;
    boolean needSetCookie = false;
    //check request changing preferences:
    String lang = pRequestData.getParameter("lang");
    String decSep = pRequestData.getParameter("decSep");
    String decGrSep = pRequestData.getParameter("decGrSep");
    String digInGr = pRequestData.getParameter("digInGr");
    if (pIsFirstReq || isShowDebMsg && detLev > 60000) {
      this.logger.debug(null, HndlI18nRequest.class,
        "Request's lang/decSep/decGrSep/digInGr: " + lang + "/"
          + decSep + "/" + decGrSep + "/" + digInGr);
    }
    if (lang == null || lang.length() == 0
      || decSep == null || decSep.length() == 0
        || decGrSep == null || decGrSep.length() == 0
          || digInGr == null || digInGr.length() == 0) {
      lang = null;
      decSep = null;
      decGrSep = null;
      digInGr = null;
    } else {
      needSetCookie = true;
    }
    if (decGrSep == null && decSep == null && lang == null) {
      lang = pRequestData.getCookieValue("lang");
      decSep = pRequestData.getCookieValue("decSep");
      decGrSep = pRequestData.getCookieValue("decGrSep");
      digInGr = pRequestData.getCookieValue("digInGr");
      if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
        this.logger.debug(null, HndlI18nRequest.class,
          "Client's cookie lang/decSep/decGrSep/digInGr: " + lang
            + "/" + decSep + "/" + decGrSep + "/" + digInGr);
      }
    }
    if (decGrSep != null && decSep != null && lang != null
      && digInGr != null) {
      if (decGrSep.equals(decSep)) {
        if (pIsFirstReq || isShowDebMsg) {
          this.logger.debug(null, HndlI18nRequest.class,
            "Separators are same!! decSep/decGrSep: " + decSep);
        }
      } else {
        //try to match from cookies or changed by user:
        lpf = new LangPreferences();
        lpf.setDigitsInGroup(Integer.parseInt(digInGr));
        for (Languages ln : pLangs) {
          if (ln.getItsId().equals(lang)) {
            lpf.setLang(ln);
            break;
          }
        }
        if (lpf.getLang() == null) {
          lpf = null;
        } else {
          for (DecimalSeparator ds : pDecSeps) {
            if (ds.getItsId().equals(decSep)) {
              lpf.setDecimalSep(ds);
              break;
            }
          }
          if (lpf.getDecimalSep() == null) {
            lpf = null;
          } else {
            for (DecimalGroupSeparator dgs : pDecGrSeps) {
              if (dgs.getItsId().equals(decGrSep)) {
                lpf.setDecimalGroupSep(dgs);
                break;
              }
            }
            if (lpf.getDecimalGroupSep() == null) {
              lpf = null;
            }
          }
        }
      }
    }
    if (lpf == null) {
      // try match client's locale, if not - default or the first:
      String ccountry = null;
      String clang = null;
      if (pRequestData.getLocale() != null) {
        ccountry = pRequestData.getLocale().getCountry();
        clang = pRequestData.getLocale().getLanguage();
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
            "Client prefers lang/country: " + clang + "/" + ccountry);
        }
      }
      LangPreferences lpfMf = null;
      LangPreferences lpfMl = null;
      LangPreferences lpfDef = null;
      for (LangPreferences lpft : pLangPrefs) {
        if (lpft.getCountry().getItsId().equals(ccountry)
          && lpft.getLang().getItsId().equals(clang)) {
          lpfMf = lpft;
          break;
        }
        if (lpft.getLang().getItsId().equals(clang)) {
          lpfMl = lpft;
        }
        if (lpft.getIsDefault()) {
          lpfDef = lpft;
        } else if (lpfDef == null) {
          lpfDef = lpft;
        }
      }
      if (lpfMf != null) {
        lpf = lpfMf;
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
        "Full match lang/decSep/decGrSep/digInGr: " + lpf.getLang()
      .getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
    + lpf.getDecimalGroupSep().getItsId() + "/" + lpf.getDigitsInGroup());
        }
      } else if (lpfMl != null) {
        lpf = lpfMl;
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
        "Lang match lang/decSep/decGrSep/digInGr: " + lpf.getLang()
      .getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
    + lpf.getDecimalGroupSep().getItsId() + "/" + lpf.getDigitsInGroup());
        }
      } else {
        lpf = lpfDef;
        if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
          this.logger.debug(null, HndlI18nRequest.class,
        "No match, default lang/decSep/decGrSep/digInGr: "
      + lpf.getLang().getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
    + lpf.getDecimalGroupSep().getItsId() + "/" + lpf.getDigitsInGroup());
        }
      }
      needSetCookie = true;
    }
    if (needSetCookie) {
      pRequestData.setCookieValue("digInGr",
        lpf.getDigitsInGroup().toString());
      pRequestData.setCookieValue("lang", lpf.getLang().getItsId());
      pRequestData.setCookieValue("decSep",
        lpf.getDecimalSep().getItsId());
      pRequestData.setCookieValue("decGrSep",
        lpf.getDecimalGroupSep().getItsId());
      if (pIsFirstReq || isShowDebMsg && detLev > 10000) {
        this.logger.debug(null, HndlI18nRequest.class,
          "Cookie is set lang/decSep/decGrSep: " + lpf.getLang()
            .getItsId() + "/" + lpf.getDecimalSep().getItsId() + "/"
              + lpf.getDecimalGroupSep().getItsId());
      }
    }
    return lpf;
  }


  /**
   * <p>Handle data changed event.</p>
   * @throws Exception - an exception
   **/
  @Override
  public final synchronized void handleDataChanged() throws Exception {
    this.langPreferences = null;
    this.languages = null;
    this.decSeps = null;
    this.decGrSeps = null;
    this.logger.info(null, HndlI18nRequest.class, "I18N changes are handled.");
    if (this.additionalI18nReqHndl != null) {
      this.additionalI18nReqHndl.handleDataChanged();
    }
  }

  //Simple getters and setters (their synchronization never hit performance):
  /**
   * <p>Geter for logger.</p>
   * @return ILogger
   **/
  public final synchronized ILogger getLogger() {
    return this.logger;
  }

  /**
   * <p>Setter for logger.</p>
   * @param pLogger reference
   **/
  public final synchronized void setLogger(final ILogger pLogger) {
    this.logger = pLogger;
  }

  /**
   * <p>Getter for srvDatabase.</p>
   * @return ISrvDatabase<RS>
   **/
  public final synchronized ISrvDatabase<RS> getSrvDatabase() {
    return this.srvDatabase;
  }

  /**
   * <p>Setter for srvDatabase.</p>
   * @param pSrvDatabase reference
   **/
  public final synchronized void setSrvDatabase(
    final ISrvDatabase<RS> pSrvDatabase) {
    this.srvDatabase = pSrvDatabase;
  }

  /**
   * <p>Getter for srvOrm.</p>
   * @return ISrvOrm<RS>
   **/
  public final synchronized ISrvOrm<RS> getSrvOrm() {
    return this.srvOrm;
  }

  /**
   * <p>Setter for srvOrm.</p>
   * @param pSrvOrm reference
   **/
  public final synchronized void setSrvOrm(final ISrvOrm<RS> pSrvOrm) {
    this.srvOrm = pSrvOrm;
  }

  /**
   * <p>Getter for additionalI18nReqHndl.</p>
   * @return IHandlerRequest
   **/
  public final IHandlerRequestDch getAdditionalI18nReqHndl() {
    return this.additionalI18nReqHndl;
  }

  /**
   * <p>Setter for additionalI18nReqHndl.</p>
   * @param pAdditionalI18nReqHndl reference
   **/
  public final void setAdditionalI18nReqHndl(
    final IHandlerRequestDch pAdditionalI18nReqHndl) {
    this.additionalI18nReqHndl = pAdditionalI18nReqHndl;
  }
}
